@@ -52,6 +52,13 @@ START_MENU = envbool("XPRA_SHOW_START_MENU", True)
 MENU_ICONS = envbool("XPRA_MENU_ICONS", True)
 
 
+def s(v):
+    try:
+        return v.decode("utf-8")
+    except AttributeError:
+        return str(v)
+
+
 def get_bandwidth_menu_options():
     options = []
     for x in os.environ.get("XPRA_BANDWIDTH_MENU_OPTIONS", "1,2,5,10,20,50,100").split(","):
@@ -451,6 +458,7 @@ class GTKTrayMenuBase(object):
             menu.append(self.make_modalwindowmenuitem())
         if mixin_features.windows and self.client.keyboard_helper:
             menu.append(self.make_keyboardsyncmenuitem())
+            menu.append(self.make_shortcutsmenuitem())
 
     def make_sharingmenuitem(self):
         def sharing_toggled(*args):
@@ -685,13 +693,12 @@ class GTKTrayMenuBase(object):
             if not c.server_clipboard:
                 self.clipboard_menuitem.set_tooltip_text("Server does not support clipboard synchronization")
                 return
-            if not c.client_supports_clipboard:
+            ch = c.clipboard_helper
+            if not c.client_supports_clipboard or not ch:
                 self.clipboard_menuitem.set_tooltip_text("Client does not support clipboard synchronization")
                 return
             #add a submenu:
             set_sensitive(self.clipboard_menuitem, True)
-            c = self.client
-            ch = self.client.clipboard_helper
             clipboard_submenu = gtk.Menu()
             self.clipboard_menuitem.set_submenu(clipboard_submenu)
             self.popup_menu_workaround(clipboard_submenu)
@@ -755,6 +762,18 @@ class GTKTrayMenuBase(object):
         self.client.after_handshake(set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
 
+    def make_shortcutsmenuitem(self):
+        self.keyboard_shortcuts_menuitem = self.checkitem("Keyboard Shortcuts")
+        kh = self.client.keyboard_helper
+        self.keyboard_shortcuts_menuitem.set_active(kh and bool(kh.shortcuts_enabled))
+        def keyboard_shortcuts_toggled(*args):
+            ks = self.keyboard_shortcuts_menuitem.get_active()
+            log("keyboard_shortcuts_toggled%s enabled=%s", args, ks)
+            kh.shortcuts_enabled = ks
+        self.keyboard_shortcuts_menuitem.connect("toggled", keyboard_shortcuts_toggled)
+        return self.keyboard_shortcuts_menuitem
+
+
     def make_openglmenuitem(self):
         gl = self.checkitem("OpenGL")
         gl.set_tooltip_text("hardware accelerated rendering using OpenGL")
@@ -1607,14 +1626,14 @@ class GTKTrayMenuBase(object):
             if not entries:
                 continue
             icondata = category_props.get(b"IconData")
-            category_menu_item = self.start_menuitem(category.decode("utf-8"), icondata)
+            category_menu_item = self.start_menuitem(s(category), icondata)
             cat_menu = gtk.Menu()
             category_menu_item.set_submenu(cat_menu)
             self.popup_menu_workaround(cat_menu)
             menu.append(category_menu_item)
             for app_name, command_props in sorted(entries.items()):
                 log("build_start_menu() app_name=%s", app_name)
-                app_menu_item = self.make_applaunch_menu_item(app_name, command_props)
+                app_menu_item = self.make_applaunch_menu_item(s(app_name), command_props)
                 cat_menu.append(app_menu_item)
         menu.show_all()
         return menu
@@ -1623,7 +1642,11 @@ class GTKTrayMenuBase(object):
         pixbuf = None
         if app_name and not icondata:
             #try to load from our icons:
-            icon_filename = os.path.join(get_icon_dir(), "%s.png" % app_name.decode("utf-8").lower())
+            try:
+                nstr = app_name.decode("utf-8").lower()
+            except UnicodeDecodeError:
+                nstr = bytestostr(app_name).lower()
+            icon_filename = os.path.join(get_icon_dir(), "%s.png" % nstr)
             if os.path.exists(icon_filename):
                 from xpra.gtk_common.gtk_util import pixbuf_new_from_file
                 pixbuf = pixbuf_new_from_file(icon_filename)
@@ -1664,7 +1687,7 @@ class GTKTrayMenuBase(object):
 
     def make_applaunch_menu_item(self, app_name, command_props):
         icondata = command_props.get(b"IconData")
-        app_menu_item = self.start_menuitem(app_name.decode("utf-8"), icondata)
+        app_menu_item = self.start_menuitem(app_name, icondata)
         def app_launch(*args):
             log("app_launch(%s) command_props=%s", args, command_props)
             command = command_props.get(b"command")
