@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2020 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -20,7 +20,7 @@ from xpra.os_util import (
     monotonic_time, bytestostr, strtobytes,
     )
 from xpra.util import (
-    flatten_dict, typedict, updict, repr_ellipsized, envint, envbool,
+    flatten_dict, typedict, updict, ellipsizer, envint, envbool,
     csv, first_time, SERVER_SHUTDOWN,
     )
 from xpra.version_util import XPRA_VERSION
@@ -36,7 +36,7 @@ PROXY_QUEUE_SIZE = envint("XPRA_PROXY_QUEUE_SIZE", 0)
 #for testing only: passthrough as RGB:
 PASSTHROUGH_RGB = envbool("XPRA_PROXY_PASSTHROUGH_RGB", False)
 VIDEO_TIMEOUT = 5                  #destroy video encoder after N seconds of idle state
-LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
+LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", False)
 PASSTHROUGH_AUTH = envbool("XPRA_PASSTHROUGH_AUTH", True)
 
 PING_INTERVAL = max(1, envint("XPRA_PROXY_PING_INTERVAL", 5))*1000
@@ -63,7 +63,7 @@ class ProxyInstance(object):
             session_options,
             video_encoder_modules,
             disp_desc, cipher, encryption_key,
-            "%s: %s.." % (type(caps), repr_ellipsized(str(caps)))))
+            "%s: %s.." % (type(caps), ellipsizer(caps))))
         self.uuid = get_hex_uuid()
         self.client_protocol = None
         self.server_protocol = None
@@ -491,6 +491,11 @@ class ProxyInstance(object):
             self.encode_queue.put(packet)
             #which will queue the packet itself when done:
             return
+        elif packet_type=="sound-data":
+            sound_data = packet[2]
+            if sound_data:
+                #best if we use raw packets for the actual sound-data chunk:
+                packet[2] = Compressed("sound-data", sound_data)
         #we do want to reformat cursor packets...
         #as they will have been uncompressed by the network layer already:
         elif packet_type=="cursor":
@@ -569,6 +574,15 @@ class ProxyInstance(object):
 
     def encode_loop(self):
         """ thread for slower encoding related work """
+        def delvideo(wid):
+            try:
+                del self.video_encoders[wid]
+            except KeyError:
+                pass
+            try:
+                del self.video_encoders_last_used_time[wid]
+            except KeyError:
+                pass
         while not self.exit:
             packet = self.encode_queue.get()
             if packet is None:
@@ -580,8 +594,7 @@ class ProxyInstance(object):
                     self.lost_windows.remove(wid)
                     ve = self.video_encoders.get(wid)
                     if ve:
-                        del self.video_encoders[wid]
-                        del self.video_encoders_last_used_time[wid]
+                        delvideo(wid)
                         ve.clean()
                 elif packet_type==b"draw":
                     #modify the packet with the video encoder:
@@ -593,14 +606,13 @@ class ProxyInstance(object):
                     wid = packet[1]
                     ve = self.video_encoders.get(wid)
                     now = monotonic_time()
-                    idle_time = now-self.video_encoders_last_used_time.get(wid)
+                    idle_time = now-self.video_encoders_last_used_time.get(wid, 0)
                     if ve and idle_time>VIDEO_TIMEOUT:
                         enclog("timing out the video encoder context for window %s", wid)
                         #timeout is confirmed, we are in the encoding thread,
                         #so it is now safe to clean it up:
                         ve.clean()
-                        del self.video_encoders[wid]
-                        del self.video_encoders_last_used_time[wid]
+                        delvideo(wid)
                 else:
                     enclog.warn("unexpected encode packet: %s", packet_type)
             except Exception:
@@ -740,9 +752,10 @@ class ProxyInstance(object):
         #to do what may need to be done if we find a timeout:
         now = monotonic_time()
         for wid in tuple(self.video_encoders_last_used_time.keys()):
-            idle_time = int(now-self.video_encoders_last_used_time.get(wid))
-            if idle_time is None:
+            vetime = self.video_encoders_last_used_time.get(wid)
+            if not vetime:
                 continue
+            idle_time = int(now-vetime)
             enclog("timeout_video_encoders() wid=%s, idle_time=%s", wid, idle_time)
             if idle_time and idle_time>VIDEO_TIMEOUT:
                 self.encode_queue.put(["check-video-timeout", wid])
