@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -49,7 +49,7 @@ from xpra.make_thread import start_thread
 from xpra.util import (
     first_time,
     csv, merge_dicts, typedict, notypedict, flatten_dict, parse_simple_dict,
-    repr_ellipsized, dump_all_frames, nonl, envint, envbool, envfloat,
+    ellipsizer, dump_all_frames, nonl, envint, envbool, envfloat,
     SERVER_SHUTDOWN, SERVER_UPGRADE, LOGIN_TIMEOUT, DONE, PROTOCOL_ERROR,
     SERVER_ERROR, VERSION_ERROR, CLIENT_REQUEST, SERVER_EXIT,
     )
@@ -72,7 +72,7 @@ main_thread = threading.current_thread()
 MAX_CONCURRENT_CONNECTIONS = envint("XPRA_MAX_CONCURRENT_CONNECTIONS", 100)
 SIMULATE_SERVER_HELLO_ERROR = envbool("XPRA_SIMULATE_SERVER_HELLO_ERROR", False)
 SERVER_SOCKET_TIMEOUT = envfloat("XPRA_SERVER_SOCKET_TIMEOUT", "0.1")
-LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
+LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", False)
 CHALLENGE_TIMEOUT = envint("XPRA_CHALLENGE_TIMEOUT", 120)
 
 ENCRYPTED_SOCKET_TYPES = os.environ.get("XPRA_ENCRYPTED_SOCKET_TYPES", "tcp,ws")
@@ -225,6 +225,7 @@ class ServerCore(object):
         self._socket_timeout = SERVER_SOCKET_TIMEOUT
         self._ws_timeout = 5
         self._socket_dir = None
+        self._socket_dirs = []
         self.dbus_pid = 0
         self.dbus_env = {}
         self.dbus_control = False
@@ -282,6 +283,7 @@ class ServerCore(object):
         self._socket_dir = opts.socket_dir or ""
         if not self._socket_dir and opts.socket_dirs:
             self._socket_dir = opts.socket_dirs[0]
+        self._socket_dirs = opts.socket_dirs
         self.encryption = opts.encryption
         self.encryption_keyfile = opts.encryption_keyfile
         self.tcp_encryption = opts.tcp_encryption
@@ -490,12 +492,15 @@ class ServerCore(object):
         dbuslog("stop_dbus_server() dbus_pid=%s", self.dbus_pid)
         if not self.dbus_pid:
             return
-        try:
-            os.kill(self.dbus_pid, signal.SIGINT)
-        except Exception as e:
-            dbuslog("os.kill(%i, SIGINT)", self.dbus_pid, exc_info=True)
-            dbuslog.warn("Warning: error trying to stop dbus with pid %i:", self.dbus_pid)
-            dbuslog.warn(" %s", e)
+        from xpra.scripts.server import add_cleanup
+        def do_kill_dbus():
+            try:
+                os.kill(self.dbus_pid, signal.SIGINT)
+            except Exception as e:
+                dbuslog("os.kill(%i, SIGINT)", self.dbus_pid, exc_info=True)
+                dbuslog.warn("Warning: error trying to stop dbus with pid %i:", self.dbus_pid)
+                dbuslog.warn(" %s", e)
+        add_cleanup(do_kill_dbus)
 
     def init_dbus_server(self):
         if not POSIX:
@@ -1085,6 +1090,8 @@ class ServerCore(object):
                         return
                     ssllog("ws socket receiving ssl, upgrading")
                     conn = ssl_wrap()
+                    if conn is None:
+                        return
                 elif len(peek_data)>=2 and peek_data[0] in ("P", ord("P") and peek_data[1] in ("\x00", 0)):
                     self.new_conn_err(conn, sock, socktype, socket_info, "xpra",
                                       "packet looks like a plain xpra packet")
@@ -1253,7 +1260,7 @@ class ServerCore(object):
             #xpra packet header, no need to wrap this connection
             return True, conn, peek_data
         frominfo = pretty_socket(conn.remote)
-        netlog("may_wrap_socket(..) peek_data=%s from %s", repr_ellipsized(bytestostr(peek_data)), frominfo)
+        netlog("may_wrap_socket(..) peek_data=%s from %s", ellipsizer(peek_data), frominfo)
         try:
             first_char = ord(peek_data[0])
         except TypeError:
@@ -1305,7 +1312,7 @@ class ServerCore(object):
 
     def invalid_header(self, proto, data, msg=""):
         netlog("invalid_header(%s, %s bytes: '%s', %s)",
-               proto, len(data or ""), msg, repr_ellipsized(data))
+               proto, len(data or ""), msg, ellipsizer(data))
         netlog(" input_packetcount=%s, tcp_proxy=%s, html=%s, ssl=%s",
                proto.input_packetcount, self._tcp_proxy, self._html, bool(self._ssl_wrap_socket))
         info = self.guess_header_protocol(data)[1]
@@ -1318,7 +1325,7 @@ class ServerCore(object):
         except TypeError:
             c = int(v[0])
         s = bytestostr(v)
-        netlog("guess_header_protocol(%r) first char=%#x", repr_ellipsized(s), c)
+        netlog("guess_header_protocol(%r) first char=%#x", ellipsizer(s), c)
         if c==0x16:
             return "ssl", "SSL packet?"
         if s[:4]=="SSH-":
@@ -1359,7 +1366,7 @@ class ServerCore(object):
                      daemon=True, args=(socktype, conn, is_ssl, req_info, line1, conn.remote))
 
     def start_http(self, socktype, conn, is_ssl, req_info, line1, frominfo):
-        httplog("start_http(%s, %s, %s, %s, %s, %s) www dir=%s, headers dir=%s",
+        httplog("start_http(%s, %s, %s, %s, %r, %s) www dir=%s, headers dir=%s",
                 socktype, conn, is_ssl, req_info, line1, frominfo,
                 self._www_dir, self._http_headers_dir)
         try:
@@ -1410,14 +1417,19 @@ class ServerCore(object):
         return self.send_http_response(handler, "ready")
 
     def send_http_response(self, handler, content, content_type="text/plain"):
-        handler.send_response(200)
-        headers = {
-            "Content-type"      : content_type,
-            "Content-Length"    : len(content),
-            }
-        for k,v in headers.items():
-            handler.send_header(k, v)
+        if content is None:
+            handler.send_response(404)
+        else:
+            handler.send_response(200)
+            headers = {
+                "Content-type"      : content_type,
+                "Content-Length"    : len(content),
+                }
+            for k,v in headers.items():
+                handler.send_header(k, v)
         handler.end_headers()
+        if isinstance(content, str):
+            content = content.encode("latin1")
         return content
 
 
@@ -1573,13 +1585,13 @@ class ServerCore(object):
     def _process_gibberish(self, proto, packet):
         message, data = packet[1:3]
         netlog("Received uninterpretable nonsense from %s: %s", proto, message)
-        netlog(" data: %s", repr_ellipsized(data))
+        netlog(" data: %s", ellipsizer(data))
         self.disconnect_client(proto, message)
 
     def _process_invalid(self, protocol, packet):
         message, data = packet[1:3]
         netlog("Received invalid packet: %s", message)
-        netlog(" data: %s", repr_ellipsized(data))
+        netlog(" data: %s", ellipsizer(data))
         self.disconnect_client(protocol, message)
 
 
@@ -1633,6 +1645,11 @@ class ServerCore(object):
                 auth, aclass, options = auth_class
                 opts = dict(options)
                 opts["connection"] = conn
+                def parse_socket_dirs(v):
+                    if isinstance(v, (tuple, list)):
+                        return v
+                    return str(v).split(",")
+                opts["socket-dirs"] = parse_socket_dirs(opts.get("socket-dirs", self._socket_dirs))
                 authenticator = aclass(username, **opts)
                 authlog("authenticator %i: %s(%s, %s)=%s", i, auth, username, opts, authenticator)
                 authenticators.append(authenticator)
@@ -1689,7 +1706,7 @@ class ServerCore(object):
             if padding not in ALL_PADDING_OPTIONS:
                 auth_failed("unsupported padding: %s" % padding)
                 return
-            authlog("set output cipher using encryption key '%s'", repr_ellipsized(encryption_key))
+            authlog("set output cipher using encryption key '%s'", ellipsizer(encryption_key))
             proto.set_cipher_out(cipher, cipher_iv, encryption_key, key_salt, iterations, padding)
             #use the same cipher as used by the client:
             auth_caps = new_cipher_caps(proto, cipher, encryption_key, padding_options)
@@ -1822,7 +1839,7 @@ class ServerCore(object):
                 raise Exception("Simulating a server error")
             self.hello_oked(proto, packet, c, auth_caps)
         except ClientException as e:
-            log("call_hello_oked(%s, %s, %s, %s)", proto, packet, repr_ellipsized(str(c)), auth_caps, exc_info=True)
+            log("call_hello_oked(%s, %s, %s, %s)", proto, packet, ellipsizer(c), auth_caps, exc_info=True)
             log.error("Error setting up new connection for")
             log.error(" %s:", proto)
             log.error(" %s", e)
@@ -1845,13 +1862,13 @@ class ServerCore(object):
         if is_req("id"):
             self.send_id_info(proto)
             return True
+        if self._closing:
+            self.disconnect_client(proto, SERVER_EXIT, "server is shutting down")
+            return True
         if is_req("info"):
             flatten = not c.boolget("info-namespace", False)
             self.send_hello_info(proto, flatten)
             return True
-        if self._closing:
-            self.disconnect_client(proto, SERVER_EXIT, "server is shutting down")
-            return True
         return False
 
 
@@ -1859,7 +1876,7 @@ class ServerCore(object):
         #max packet size from client (the biggest we can get are clipboard packets)
         netlog("accept_client(%s, %s)", proto, c)
         #note: when uploading files, we send them in chunks smaller than this size
-        proto.max_packet_size = 1024*1024  #1MB
+        proto.max_packet_size = 16*1024*1024  #16MB
         proto.parse_remote_caps(c)
         self.accept_protocol(proto)
 
