@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,10 +12,90 @@ from xpra.platform.dotxpra import norm_makepath
 from xpra.scripts.config import InitException
 
 
+def source_env(source=()):
+    log = get_util_logger()
+    env = {}
+    for f in source:
+        e = env_from_sourcing(f)
+        log("source_env %s=%s", f, e)
+        env.update(e)
+    return env
+
+
+# credit: https://stackoverflow.com/a/47080959/428751
+# returns a dictionary of the environment variables resulting from sourcing a file
+def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
+    import json
+    import subprocess
+    from xpra.log import Logger
+    log = Logger("exec")
+    filename = file_to_source_path
+    if not os.path.exists(file_to_source_path):
+        filename = which(file_to_source_path)
+        if not filename:
+            log.error("Error: cannot find file '%s' to source", file_to_source_path)
+            return {}
+    if not os.path.isabs(filename):
+        filename = os.path.abspath(filename)
+    #figure out if this is a script to source,
+    #or if we're meant to execute it directly
+    try:
+        with open(filename, 'rb') as f:
+            first_line = f.readline()
+    except OSError as e:
+        log.error("Error: failed to read from '%s'", filename)
+        log.error(" %s", e)
+        first_line = b""
+    if first_line.startswith(b"\x7fELF") or b"\x00" in first_line:
+        cmd = [filename]
+        def decode(out):
+            env = {}
+            for line in out.splitlines():
+                parts = line.split("=", 1)
+                if len(parts)==2:
+                    env[parts[0]] = parts[1]
+            log("decode(%r)=%s", out, env)
+            return env
+    else:
+        source = '%ssource %s' % ("set -a && " if include_unexported_variables else "", filename)
+        dump = 'python%i.%i -c "import os, json;print(json.dumps(dict(os.environ)))"' % (sys.version_info.major, sys.version_info.minor)
+        cmd = ['/bin/bash', '-c', '%s && %s' % (source, dump)]
+        def decode(out):
+            try:
+                env = json.loads(out)
+                log("json loads(%r)=%s", out, env)
+                return env
+            except json.decoder.JSONDecodeError:
+                log.error("Error decoding json output from sourcing script '%s'",
+                          file_to_source_path, exc_info=True)
+                return {}
+    try:
+        log("env_from_sourcing%s cmd=%s", (filename, include_unexported_variables), cmd)
+        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+        out = proc.communicate()[0]
+        if proc.returncode!=0:
+            log.error("Error %i running source script '%s'", proc.returncode, filename)
+    except OSError as e:
+        log("env_from_sourcing%s", (filename, include_unexported_variables), exc_info=True)
+        log(" stdout=%r (%s)", out, type(out))
+        log.error("Error running source script '%s'", proc.returncode, file_to_source_path)
+        log.error(" %s", e)
+        return {}
+    log("stdout(%s)=%r", filename, out)
+    if not out:
+        return {}
+    try:
+        out = out.decode()
+    except UnicodeDecodeError:
+        log.error("Error decoding output from '%s'", filename, exc_info=True)
+        return {}
+    return decode(out)
+
+
 if PYTHON3:
     def sh_quotemeta(s):
         return b"'" + s.replace(b"'", b"'\\''") + b"'"
-    
+
     def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
         script = []
         script.append(b"#!/bin/sh\n")
@@ -83,18 +163,18 @@ if PYTHON3:
             script.append(b"_XPRA_PYTHON=%s\n" % (sh_quotemeta(sys.executable.encode()),))
             script.append(b"_XPRA_SCRIPT=%s\n" % (sh_quotemeta(xpra_file.encode()),))
             script.append(b"""
-    if which "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
-        # Happypath:
-        exec "$_XPRA_PYTHON" "$_XPRA_SCRIPT" "$@"
-    else
-        cat >&2 <<END
-        Could not find one or both of '$_XPRA_PYTHON' and '$_XPRA_SCRIPT'
-        Perhaps your environment has changed since the xpra server was started?
-        I'll just try executing 'xpra' with current PATH, and hope...
-    END
-        exec xpra "$@"
-    fi
-    """)
+if which "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
+    # Happypath:
+    exec "$_XPRA_PYTHON" "$_XPRA_SCRIPT" "$@"
+else
+    cat >&2 <<END
+    Could not find one or both of '$_XPRA_PYTHON' and '$_XPRA_SCRIPT'
+    Perhaps your environment has changed since the xpra server was started?
+    I'll just try executing 'xpra' with current PATH, and hope...
+END
+    exec xpra "$@"
+fi
+""")
         return b"".join(script)
 
 else:
@@ -169,18 +249,18 @@ else:
             script.append("_XPRA_PYTHON=%s\n" % (sh_quotemeta(sys.executable),))
             script.append("_XPRA_SCRIPT=%s\n" % (sh_quotemeta(xpra_file),))
             script.append("""
-    if which "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
-        # Happypath:
-        exec "$_XPRA_PYTHON" "$_XPRA_SCRIPT" "$@"
-    else
-        cat >&2 <<END
-        Could not find one or both of '$_XPRA_PYTHON' and '$_XPRA_SCRIPT'
-        Perhaps your environment has changed since the xpra server was started?
-        I'll just try executing 'xpra' with current PATH, and hope...
-    END
-        exec xpra "$@"
-    fi
-    """)
+if which "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
+    # Happypath:
+    exec "$_XPRA_PYTHON" "$_XPRA_SCRIPT" "$@"
+else
+    cat >&2 <<END
+    Could not find one or both of '$_XPRA_PYTHON' and '$_XPRA_SCRIPT'
+    Perhaps your environment has changed since the xpra server was started?
+    I'll just try executing 'xpra' with current PATH, and hope...
+END
+    exec xpra "$@"
+fi
+""")
         return "".join(script).encode()
 
 def write_runner_shell_scripts(contents, overwrite=True):
