@@ -17,6 +17,7 @@ from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImpo
 from xpra.gtk_common.error import XError, xswallow, xsync, xlog, trap, verify_sync
 from xpra.gtk_common.gtk_util import get_xwindow, display_get_default, get_default_root_window
 from xpra.server.server_uuid import save_uuid, get_uuid
+from xpra.server.source.windows_mixin import WindowsMixin
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
 from xpra.x11.gtk_x11.prop import prop_get, prop_set
 from xpra.x11.gtk_x11.gdk_display_source import close_gdk_display_source
@@ -94,6 +95,7 @@ class X11ServerCore(GTKServerBase):
         self.keys_pressed = {}
         self.last_mouse_user = None
         self.x11_filter = False
+        self.randr_sizes_added = []
         GTKServerBase.__init__(self)
         log("XShape=%s", X11Window.displayHasXShape())
 
@@ -367,7 +369,7 @@ class X11ServerCore(GTKServerBase):
                     "touchpad-device"              : bool(self.touchpad_device),
                     })
             if self.randr:
-                sizes = RandR.get_xrr_screen_sizes()
+                sizes = self.get_all_screen_sizes()
                 if len(sizes)>1:
                     capabilities["screen-sizes"] = sizes
             if self.default_cursor_image and source.wants_default_cursor:
@@ -421,7 +423,7 @@ class X11ServerCore(GTKServerBase):
         #randr:
         if self.randr:
             with xlog:
-                sizes = RandR.get_xrr_screen_sizes()
+                sizes = self.get_all_screen_sizes()
                 if len(sizes)>=0:
                     sinfo["randr"] = {
                         ""          : True,
@@ -534,10 +536,19 @@ class X11ServerCore(GTKServerBase):
         return (cursor_image, cursor_sizes)
 
 
+    def get_all_screen_sizes(self):
+        #workaround for #2910: the resolutions we add are not seen by XRRSizes!
+        # so we keep track of the ones we have added ourselves:
+        sizes = list(RandR.get_xrr_screen_sizes())
+        for w, h in self.randr_sizes_added:
+            if (w, h) not in sizes:
+                sizes.append((w, h))
+        return tuple(sizes)
+
     def get_max_screen_size(self):
         max_w, max_h = self.root_window.get_geometry()[2:4]
         if self.randr:
-            sizes = RandR.get_xrr_screen_sizes()
+            sizes = self.get_all_screen_sizes()
             if self.randr and len(sizes)>=1:
                 for w,h in sizes:
                     max_w = max(max_w, w)
@@ -554,7 +565,7 @@ class X11ServerCore(GTKServerBase):
         root_w, root_h = self.root_window.get_geometry()[2:4]
         if not self.randr:
             return root_w, root_h
-        sss = self._server_sources.values()
+        sss = tuple(x for x in self._server_sources.values() if x.ui_client)
         if len(sss)>1:
             screenlog.info("screen used by %i clients:", len(sss))
         bigger = True
@@ -595,7 +606,7 @@ class X11ServerCore(GTKServerBase):
     def do_get_best_screen_size(self, desired_w, desired_h, bigger=True):
         if not self.randr:
             return desired_w, desired_h
-        screen_sizes = RandR.get_xrr_screen_sizes()
+        screen_sizes = self.get_all_screen_sizes()
         if (desired_w, desired_h) in screen_sizes:
             return desired_w, desired_h
         if self.randr_exact_size:
@@ -606,6 +617,7 @@ class X11ServerCore(GTKServerBase):
                         #we have to wait a little bit
                         #to make sure that everything sees the new resolution
                         #(ideally this method would be split in two and this would be a callback)
+                        self.randr_sizes_added.append(v)
                         import time
                         time.sleep(0.5)
                         return v
@@ -613,7 +625,7 @@ class X11ServerCore(GTKServerBase):
                 screenlog.warn("Warning: failed to add resolution %ix%i:", desired_w, desired_h)
                 screenlog.warn(" %s", e)
             #re-query:
-            screen_sizes = RandR.get_xrr_screen_sizes()
+            screen_sizes = self.get_all_screen_sizes()
         #try to find the best screen size to resize to:
         new_size = None
         closest = {}
@@ -707,7 +719,7 @@ class X11ServerCore(GTKServerBase):
                 #the change! (ugly! but this works)
                 with xsync:
                     temp = {}
-                    for tw,th in RandR.get_xrr_screen_sizes():
+                    for tw,th in self.get_all_screen_sizes():
                         if tw!=w or th!=h:
                             #use the number of extra pixels as key:
                             #(so we can choose the closest resolution)
@@ -731,7 +743,7 @@ class X11ServerCore(GTKServerBase):
                 #we can use XRRSetScreenSize:
                 try:
                     with xsync:
-                        RandR.xrr_set_screen_size(w, h, self.xdpi or self.dpi, self.ydpi or self.dpi)
+                        RandR.xrr_set_screen_size(w, h, self.xdpi or self.dpi or 96, self.ydpi or self.dpi or 96)
                 except XError:
                     screenlog("XRRSetScreenSize failed", exc_info=True)
             screenlog("calling RandR.get_screen_size()")
@@ -848,8 +860,9 @@ class X11ServerCore(GTKServerBase):
         #so we use wid=0 for that:
         wid = 0
         for ss in self._server_sources.values():
-            name = strtobytes(event.bell_name or "")
-            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
+            if isinstance(ss, WindowsMixin):
+                name = strtobytes(event.bell_name or "")
+                ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
 
 
     def _bell_signaled(self, wm, event):
